/*
 *
 * Made by Jiri Stepanovsky, jiristepanovsky.cz
 *
 * Created 23/9/2014
 *
 * Part of EV3 API
 * Responsible for controlling sensors on EV3
 *
 */

#include "ev3_input.h"

// Temporary definitions - https://github.com/ev3dev/ev3dev/wiki/Using-Sensors
#define DEVICE_CODE_TOUCH 16
#define DEVICE_CODE_COLOR 29
#define DEVICE_CODE_SONAR 30
#define DEVICE_CODE_GYRO 32

/*
 * A lot of abolutlely usesless variables reqired for RAM mapping
 * Couse their size must correspond to size of device files in EV3
 *
 * This piece of code has been copied mainly from /lms2012/lms2012.c
 * secondary from /lms2012/*.*
 */
 
//#define   DISABLE_FAST_DATALOG_BUFFER    // <- Do not define
//#define   DISABLE_OLD_COLOR   // <- Do not define
#define   DISABLE_PREEMPTED_VM   // <- Originaly defined, no reason to not doing it
#define   COLORS 4   // <- Colors, no idea why 4, when fourth color is not used
#define   CALPOINTS 3    // <- Calibration values for 3 color recognizing (fourth not used)
typedef   struct {
  unsigned long    Calibration[CALPOINTS][COLORS];
  unsigned short   CalLimits[CALPOINTS - 1];
  unsigned short   Crc;
  unsigned short   ADRaw[COLORS];
  unsigned short   SensorRaw[COLORS];
} COLORSTRUCT;    // <- Datatype for old NXT Color sensor
typedef   struct { // if data type changes - remember to change "cInputTypeDataInit" !
  signed char     Name[TYPE_NAME_LENGTH + 1]; //!< Device name
  char     Type;                       //!< Device type
  char     Connection;
  char     Mode;                       //!< Device mode
  char     DataSets;
  char     Format;
  char     Figures;
  char     Decimals;
  char     Views;
  float     RawMin;                     //!< Raw minimum value      (e.c. 0.0)
  float     RawMax;                     //!< Raw maximum value      (e.c. 1023.0)
  float     PctMin;                     //!< Percent minimum value  (e.c. -100.0)
  float     PctMax;                     //!< Percent maximum value  (e.c. 100.0)
  float     SiMin;                      //!< SI unit minimum value  (e.c. -100.0)
  float     SiMax;                      //!< SI unit maximum value  (e.c. 100.0)
  unsigned short     InvalidTime;                //!< mS from type change to valid data
  unsigned short     IdValue;                    //!< Device id value        (e.c. 0 ~ UART)
  char     Pins;                       //!< Device pin setup
  signed char     Symbol[SYMBOL_LENGTH + 1];  //!< SI unit symbol
  unsigned short     Align;
} TYPES;    // Data struct for complete description of 1 device
typedef   struct {
  short  InPin1[NUM_INPUTS];         //!< Analog value at input port connection 1
  short  InPin6[NUM_INPUTS];         //!< Analog value at input port connection 6
  short  OutPin5[NUM_OUTPUTS];       //!< Analog value at output port connection 5
  short  BatteryTemp;            //!< Battery temperature
  short  MotorCurrent;           //!< Current flowing to motors
  short  BatteryCurrent;         //!< Current flowing from the battery
  short  Cell123456;             //!< Voltage at battery cell 1, 2, 3,4, 5, and 6
#ifndef DISABLE_FAST_DATALOG_BUFFER
  short  Pin1[NUM_INPUTS][DEVICE_LOGBUF_SIZE];      //!< Raw value from analog device
  short  Pin6[NUM_INPUTS][DEVICE_LOGBUF_SIZE];      //!< Raw value from analog device
  unsigned short   Actual[NUM_INPUTS];
  unsigned short   LogIn[NUM_INPUTS];
  unsigned short   LogOut[NUM_INPUTS];
#endif
#ifndef   DISABLE_OLD_COLOR
  COLORSTRUCT  NxtCol[NUM_INPUTS];
#endif
  short  OutPin5Low[NUM_OUTPUTS];    //!< Analog value at output port connection 5 when connection 6 is low
  char   Updated[NUM_INPUTS];
  char   InDcm[NUM_INPUTS];          //!< Input port device types
  char   InConn[NUM_INPUTS];
  char   OutDcm[NUM_OUTPUTS];        //!< Output port device types
  char   OutConn[NUM_OUTPUTS];
#ifndef DISABLE_PREEMPTED_VM
  unsigned short   PreemptMilliSeconds;
#endif
} ANALOG;    // <- Data type for reading analog values from system (battery, etc.) and touch sensors
typedef   struct{
  TYPES   TypeData[NUM_INPUTS][MAX_DEVICE_MODES]; //!< TypeData
#ifndef DISABLE_FAST_DATALOG_BUFFER
  unsigned short   Repeat[NUM_INPUTS][DEVICE_LOGBUF_SIZE];
  char   Raw[NUM_INPUTS][DEVICE_LOGBUF_SIZE][UART_DATA_LENGTH];      //!< Raw value from UART device
  unsigned short   Actual[NUM_INPUTS];
  unsigned short   LogIn[NUM_INPUTS];
#else
  char   Raw[NUM_INPUTS][UART_DATA_LENGTH];      //!< Raw value from UART device
#endif
  char   Status[NUM_INPUTS];                     //!< Status
  char   Output[NUM_INPUTS][UART_DATA_LENGTH];   //!< Bytes to UART device
  char   OutputLength[NUM_INPUTS];
} UART;    // Data type consisting mainly from values from uart-compatible sensors (color, sonar, etc.)
typedef   struct{
  char     Connection[NUM_INPUTS];
  char     Type[NUM_INPUTS];
  char     Mode[NUM_INPUTS];
} DEVCON;    // Devices configuration


/*
 * End of copied code
 *
 */

// All variables used in Input EV3 API
typedef struct {
    int analogFile;
    int uartFile;
    ANALOG Analog;
    ANALOG *pAnalog;
    UART *pUart;
    DEVCON DevCon;
} InputGlobals;
InputGlobals InputInstance;
bool InputBoolInitialized = false;
int InputSensorInitDone = 0;

// Write device configurations
bool InputSensorsDevConInit(){
    if(InputBoolInitialized)
        return false;
    // define default value to not defined inputs
    int port;
    for(port = 0; port < NUM_INPUTS; port++){
        if(!(InputSensorInitDone&(1<<port))){
            InputInstance.DevCon.Mode[port] = 0; // <- no mode
            InputInstance.DevCon.Connection[port] = CONN_INPUT_DUMB; // <- Dumb sensor
            InputInstance.DevCon.Type[port] = 126; // <- no device
        }
    }
    InputSensorInitDone = 2;
    while(InputSensorInitDone > 0){
        ioctl(InputInstance.uartFile, _IOWR('u',0,DEVCON), &InputInstance.DevCon);
        Wait(50);
        InputSensorInitDone--;
    }
    //write(InputInstance.uartFile, &InputInstance.DevCon, sizeof(DEVCON));
    InputBoolInitialized = true;
    return true;
}

// return uart raw data
char InputGetUartByte(int port, int byteNum){
    if(!InputBoolInitialized)
        return 0;
    return (*InputInstance.pUart).Raw[port][(*InputInstance.pUart).Actual[port]][byteNum];
}

// Get input initializet state
bool InputInitialized(void){
    return InputBoolInitialized;
}

// Initialize input
bool InputInit(void){
     if(InputBoolInitialized)
         return true;
         
    //INITIALIZE DEVICE
    
    // To ensure that pAnalog is never uninitialized
    InputInstance.pAnalog = &InputInstance.Analog;
    
    // Open analog
    InputInstance.analogFile = open(LMS_ANALOG_DEVICE_NAME, O_RDWR | O_SYNC);
    if(InputInstance.analogFile >= 0){

        // Map analog
        InputInstance.pAnalog = (ANALOG*)mmap(0, sizeof(ANALOG), PROT_READ | PROT_WRITE, MAP_FILE | MAP_SHARED, InputInstance.analogFile, 0);
        if (InputInstance.pAnalog == MAP_FAILED) {
            close(InputInstance.analogFile);
            return false;
        } else {
        
            // Open uart
            InputInstance.uartFile = open(LMS_UART_DEVICE_NAME, O_RDWR | O_SYNC);
            if(InputInstance.uartFile >= 0){
        
                // Map uart
               	InputInstance.pUart = (UART*)mmap(0, sizeof(UART), PROT_READ | PROT_WRITE, MAP_FILE | MAP_SHARED, InputInstance.uartFile, 0);
	        if (InputInstance.pUart == MAP_FAILED){
                    munmap(InputInstance.pAnalog, sizeof(InputInstance.Analog));
                    close(InputInstance.analogFile);
                    close(InputInstance.uartFile);
	            return false;
                } else {
                
                    // Initialize successful
                    return InputSensorsDevConInit();
                }
            } else {
                munmap(InputInstance.pAnalog, sizeof(InputInstance.Analog));
                close(InputInstance.analogFile);
            }
        }
    }
    return false;
}

// Close input
bool InputExit(void){
    if (!InputBoolInitialized)
        return true;
        
    // Close analog input device
    if (InputInstance.analogFile >= 0){
        munmap(InputInstance.pAnalog, sizeof(ANALOG));
        close(InputInstance.analogFile);
        InputInstance.analogFile = -1;
    }
    // Close uart input device
    if (InputInstance.uartFile >= 0){
        munmap(InputInstance.pUart, sizeof(UART));
        close(InputInstance.uartFile);
        InputInstance.uartFile = -1;
    }
    InputBoolInitialized = false;
    return true;
}

/*
 * Sets devices configuration - You must do it before Init!!!!!!!!!
 *
 * port - input port which device is connected to (1-4)
 * type - 0: Touch sensor / 1: Color sensor / 2: Sonar sensor / 3: Gyro sensor
 * mode - mode in which the sensor will be working, depends on sensor type
 */
bool InputSetupSensor(int port, int type, int mode){
    if (InputBoolInitialized || port < 1 || port > NUM_INPUTS || type < 0 || type > 3 || mode < 0)
        return false;
    port--;
    if(type > 0){
        InputInstance.DevCon.Connection[port] = CONN_INPUT_UART; // <- Smart sensor - uart connection
    } else {
        InputInstance.DevCon.Connection[port] = CONN_INPUT_DUMB; // <- Dumb sensor - analog connection
    }
    // Parse type
    switch(type){
        case 0: if(mode > 1) return false; type = DEVICE_CODE_TOUCH; break; // Touch
        case 1: if(mode > 4) return false; type = DEVICE_CODE_COLOR; break; // Color
        case 2: if(mode > 2) return false; type = DEVICE_CODE_SONAR; break; // Sonar
        case 3: if(mode > 3 || mode == 2) return false; type = DEVICE_CODE_GYRO; break; // Gyro
        default: return false; break;
    }
    InputInstance.DevCon.Mode[port] = (char)mode; // <- Set device mode
    InputInstance.DevCon.Type[port] = (char)type; // COLOR_SENSOR_TYPE <- This instruction has no effect in the code
    InputSensorInitDone |= 1<<port;
}

// Get battery voltage in not specified units...
int InputGetBatteryVoltage(){
    if (!InputBoolInitialized)
        return 0;
    return (int)(unsigned char)((*InputInstance.pAnalog).Cell123456);
}

// Get battery temperature
int InputGetBatteryTemperature(){
    if (!InputBoolInitialized)
        return 0;
    return (int)(unsigned char)((*InputInstance.pAnalog).BatteryTemp);
}

// Get current floating from battery (probably mA)
int InputGetCurrentFromBattery(){
    if (!InputBoolInitialized)
        return 0;
    return (int)(unsigned char)((*InputInstance.pAnalog).BatteryCurrent);
}

// Get current floating to motors (probably mA)
int InputGetCurrentToMotors(){
    if (!InputBoolInitialized)
        return 0;
    return (int)(unsigned char)((*InputInstance.pAnalog).MotorCurrent);
}

/*
 * Returns input device type on port (1-4)
 * values do not correspond to reality
 */
/*
int InputDeviceType(int port){
    if (!InputBoolInitialized || port < 1 || port > NUM_INPUTS)
        return 0;
    port--;
    return (int)(unsigned char)((*InputInstance.pAnalog).InDcm[port]);
}*/

/*
 * Returns input device connection type on port (1-4)
 * values do not correspond to reality
 */
/*
int InputDeviceConnection(int port){
    if (!InputBoolInitialized || port < 1 || port > NUM_INPUTS)
        return 0;
    port--;
    return (int)(unsigned char)((*InputInstance.pAnalog).InConn[port]);
}*/

// Get value of sensor on port (1-4)
int InputGetValue(int port){
    if (!InputBoolInitialized || port < 1 || port > NUM_INPUTS)
        return 0;
    port--;
    int value = 0;
    int value2;

    // check type of sensor
    if(InputInstance.DevCon.Type[port] < 10 || InputInstance.DevCon.Type[port] > 50)
        return 0; // <- sersor is not setup

    // TOUCH
    if(InputInstance.DevCon.Type[port] == DEVICE_CODE_TOUCH){
        if(InputInstance.DevCon.Mode[port] == INPUT_SENSOR_TYPE_TOUCH_MODE_DIGITAL){
            value = (short)((*InputInstance.pAnalog).Pin6[port][(*InputInstance.pAnalog).Actual[port]]);
            if(value > 2800)
                return 1;
        } else if(InputInstance.DevCon.Mode[port] == INPUT_SENSOR_TYPE_TOUCH_MODE_ANALOG){
            return (short)((*InputInstance.pAnalog).Pin6[port][(*InputInstance.pAnalog).Actual[port]]);
        }
        return 0;
    }
    
    // COLOR
    if(InputInstance.DevCon.Type[port] == DEVICE_CODE_COLOR){
        if(InputInstance.DevCon.Mode[port] == INPUT_SENSOR_TYPE_COLOR_MODE_REFLECT){
            return (unsigned char)InputGetUartByte(port, 0);
        } else if(InputInstance.DevCon.Mode[port] == INPUT_SENSOR_TYPE_COLOR_MODE_AMBIENT){
            return (unsigned char)InputGetUartByte(port, 0);
        } else if(InputInstance.DevCon.Mode[port] == INPUT_SENSOR_TYPE_COLOR_MODE_COLOR){
            return (unsigned char)InputGetUartByte(port, 0);
        } else if(InputInstance.DevCon.Mode[port] == INPUT_SENSOR_TYPE_COLOR_MODE_REFLECT_AMBIENT){
            value = (unsigned char)InputGetUartByte(port, 0);
            value |= ((unsigned char)InputGetUartByte(port, 1)<<8);
            value |= ((unsigned char)InputGetUartByte(port, 2)<<16);
            value |= ((unsigned char)InputGetUartByte(port, 3)<<24);
            return value;
        } else if(InputInstance.DevCon.Mode[port] == INPUT_SENSOR_TYPE_COLOR_MODE_RGB_COLOR){
            value = (unsigned char)InputGetUartByte(port, 0);
            value |= ((unsigned char)InputGetUartByte(port, 1)<<8);
            if(value > 1023) value = 1023;
            value2 = (unsigned char)InputGetUartByte(port, 2);
            value2 |= ((unsigned char)InputGetUartByte(port, 3)<<8);
            if(value2 > 1023) value2 = 1023;
            value |= value2<<10;
            value2 = (unsigned char)InputGetUartByte(port, 4);
            value2 |= ((unsigned char)InputGetUartByte(port, 5)<<8);
            if(value2 > 1023) value2 = 1023;
            value |= value2<<20;
            return value;
        }
        return 0;
    }
    
    // SONAR
    if(InputInstance.DevCon.Type[port] == DEVICE_CODE_SONAR){
        if(InputInstance.DevCon.Mode[port] == INPUT_SENSOR_TYPE_SONAR_MODE_DIST_CM){
            value = (unsigned char)InputGetUartByte(port, 0);
            value |= ((unsigned char)InputGetUartByte(port, 1)<<8);
            return (signed short)value;
        } else if(InputInstance.DevCon.Mode[port] == INPUT_SENSOR_TYPE_SONAR_MODE_DIST_IN){
            value = (unsigned char)InputGetUartByte(port, 0);
            value |= ((unsigned char)InputGetUartByte(port, 1)<<8);
            return (signed short)value;
        } else if(InputInstance.DevCon.Mode[port] == INPUT_SENSOR_TYPE_SONAR_MODE_SOUND){
            return (unsigned char)InputGetUartByte(port, 0);
        }
        return 0;
    }
    
    // GYRO
    if(InputInstance.DevCon.Type[port] == DEVICE_CODE_GYRO){
        if(InputInstance.DevCon.Mode[port] == INPUT_SENSOR_TYPE_GYRO_MODE_ANG){
            value = (unsigned char)InputGetUartByte(port, 0);
            value |= ((unsigned char)InputGetUartByte(port, 1)<<8);
            return (signed short)value;
        } else if(InputInstance.DevCon.Mode[port] == INPUT_SENSOR_TYPE_GYRO_MODE_RATE){
            value = (unsigned char)InputGetUartByte(port, 0);
            value |= ((unsigned char)InputGetUartByte(port, 1)<<8);
            return (signed short)value;
        } else if(InputInstance.DevCon.Mode[port] == INPUT_SENSOR_TYPE_GYRO_MODE_ANG_RATE){
            value = (unsigned char)InputGetUartByte(port, 0);
            value |= ((unsigned char)InputGetUartByte(port, 1)<<8);
            value |= ((unsigned char)InputGetUartByte(port, 2)<<16);
            value |= ((unsigned char)InputGetUartByte(port, 3)<<24);
            return value;
        }
        return 0;
    }
    return 0;
}
