/*
 *
 * Made by Jiri Stepanovsky, jiristepanovsky.cz
 *
 * Created 23/9/2014
 *
 * Part of EV3 API
 * Responsible for controlling sensors on EV3
 *
 */

#ifdef __cplusplus
extern "C" {
#endif

#ifndef ev3_input_h
#define ev3_input_h

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <sys/mman.h>
#include <stdbool.h>
#include <limits.h>
#include  <sys/ioctl.h>

#include "ev3_constants.h"

// Init input first
bool InputInit(void);

// Exit input at the end
bool InputExit(void);

// Returns initialized state
bool InputInitialized(void);

/*
 * Sets device configuration - You must do it before Init!!!!!!!!!
 *
 * port - input port which device is connected to (1-4)
 * type - 0: Touch sensor / 1: Color sensor / 2: Sonar sensor / 3: Gyro sensor
 * mode - mode in which the sensor will be working, depends on sensor type
 */
bool InputSetupSensor(int port, int type, int mode);

// Returns voltage on battery (in some strange units)
int InputGetBatteryVoltage();

// Get battery temperature
int InputGetBatteryTemperature();

// Returns current flowing from the battery (probably miliampers)
int InputGetCurrentFromBattery();

// Returns current flowing to the motors (probably miliampers)
int InputGetCurrentToMotors();

//int InputDeviceType(int port);
//int InputDeviceConnection(int port);

// Get value of sensor on port (1-4)
int InputGetValue(int port);

#endif // ev3_input_h

#ifdef __cplusplus
}
#endif
