#include <stdio.h>
#include <unistd.h>

#include "ev3_lcd.h"
#include "ev3_command.h"
#include "ev3_button.h"
#include "ev3_input.h"

int main(void){

    // Setup sensors, required to do before calling InputInit()
    // more constants in ./ev3_constants.h
    InputSetupSensor(IN_1, INPUT_SENSOR_TYPE_TOUCH, INPUT_SENSOR_TYPE_TOUCH_MODE_ANALOG);
    InputSetupSensor(IN_2, INPUT_SENSOR_TYPE_COLOR, INPUT_SENSOR_TYPE_COLOR_MODE_COLOR);
    InputSetupSensor(IN_3, INPUT_SENSOR_TYPE_SONAR, INPUT_SENSOR_TYPE_SONAR_MODE_SOUND);
    InputSetupSensor(IN_4, INPUT_SENSOR_TYPE_GYRO, INPUT_SENSOR_TYPE_GYRO_MODE_RATE);

    // Init HW
    if(!LcdInit()){
        return -1;
    }
    if(!ButtonLedInit()){
        LcdText(1, 10, 10, "UI Init Error");
        LcdRefresh();
        LcdExit();
        return -2;
    }
    // Init Input API (required)
    if(!InputInit()){
        LcdText(1, 10, 10, "Input Init Error");
        LcdRefresh();
        SetLedPattern(LED_RED_FLASH);
        ButtonWaitForPressAndReleae(BUTTON_ID_ESCAPE);
        ButtonLedExit();
        LcdExit();
        return -3;
    }
    SetLedPattern(LED_ORANGE);

    int value;

    // press escape button for quit loop
    while(ButtonIsUp(BUTTON_ID_ESCAPE)){
        LcdClearDisplay();

        // More functions from input api can be found in ev3_input.h
        
        // Show input values - int InputGetValue(int port);
        LcdNumber(1, 50, 5, InputGetValue(IN_1));
        LcdNumber(1, 50, 25, InputGetValue(IN_2));
        LcdNumber(1, 50, 45, InputGetValue(IN_3));
        LcdNumber(1, 50, 65, InputGetValue(IN_4));
        //LcdNumber(1, 50, 85, InputGetCurrentFromBattery());
        //LcdNumber(1, 50, 105, InputGetBatteryVoltage());
        LcdText(1, 0, 105, "Press Escape for exit");
        
        LcdRefresh();
        Wait(10);
    }
    
    // Exit
    SetLedPattern(LED_GREEN);
    LcdClearDisplay();
    LcdText(1, 10, 10, "Exit");
    LcdRefresh();
    LcdExit();
    InputExit(); // EXIT Input api
    ButtonLedExit();
    return 0;
}
